import bpy
import mathutils

print("~~~~~")

#### USAGE:
#### Select all objects to be affected.
#### You needn't select the control-empties; they should be automatically detected
#### Then run the script!

for obj in bpy.context.selected_objects:
     # Ignore any control-empties included in the selection
    if obj.type == "EMPTY":
        continue
    
    # Retrieve the order in which this part is to be "constructed"
    order = obj.game.properties.get("order", None)
    if order is None:
        order = 0
    else:
        order = order.value
    
    # Get the mesh-data
    mesh = obj.data
    
    # If the mesh lacks sufficient vertex-colour layers, add them
    while len(mesh.vertex_colors) < 2:
        mesh.vertex_colors.new()
    
    # Get the relevant vertex-colour layers.
    # This script assumes that the data to be used as the alpha-channel
    # within Panda is within Blender stored in the red-channel of the second
    # layer of vertex-colours
    vColoursDist = mesh.vertex_colors[0]
    vColoursOrder = mesh.vertex_colors[1]
    
    # Determine the starting-point for this part
    # In short, this is the position of the corresponding
    # control-empty, which is simply the first empty amongst
    # the part's immediate children.
    startPt = None
    for child in obj.children:
        if child.type == "EMPTY":
            # Get the location of the empty relative to the part
            startPt = child.matrix_local*mathutils.Vector((0, 0, 0))
            continue
    if startPt is None:
        startPt = mathutils.Vector((0, 0, 0))
    
    # Now, iterate over the polygons in order to colour them
    for poly in mesh.polygons:
        # Find the polygon's centre
        centre = poly.center
        # Get the vector between the start-point and the polygon-centre
        diff = mathutils.Vector(centre) - startPt
        # And use that vector's length to calculate a value.
        # The "5" in this calculation is arbitrary, based on the model;
        # a later implementation might want to come up with a better way
        # of calculating a useful value.
        perc = diff.length/5
        
        # Finally, loop over the polygon's vertex-indices and apply relevant colours to
        # the vertex-colour layers at those indices!
        for loop_index in poly.loop_indices:
            vColoursDist.data[loop_index].color = (min(1, perc*3), min(1, (perc - 0.333333)*3), (perc - 0.6666666666)*3)
            
            vColoursOrder.data[loop_index].color = (order/64, 0, 0)
            
print ("Done!")